import os
import json
import glob

detection_patterns = {
    "Qwik": {"files": ["vite.config.ts"], "packageJson": {"dependencies": ["@builder.io/qwik"]}},
    "Analog": {"packageJson": {"dependencies": ["@analogjs/platform"]}},
    "SvelteKit": {"files": ["svelte.config.js"], "packageJson": {"dependencies": ["@sveltejs/kit"]}},
    "Fresh": {"files": ["fresh.gen.ts", "deno.json"], "contents": [{"file": "deno.json", "keywords": ["fresh"]}]},
    "Hono": {"packageJson": {"dependencies": ["hono"]}},
    "tRPC": {"packageJson": {"dependencies": ["@trpc/server"]}},
    "Prisma": {"files": ["prisma/schema.prisma"], "packageJson": {"dependencies": ["@prisma/client"]}},
    "DrizzleORM": {"packageJson": {"dependencies": ["drizzle-orm"]}},
    "TypeORM": {"packageJson": {"dependencies": ["typeorm"]}},
    "Sequelize": {"packageJson": {"dependencies": ["sequelize"]}},
    "PocketBase": {"files": ["pb_data", "pb_migrations"]},
    "React": {"files": ["src/App.jsx", "src/App.tsx", "src/index.jsx", "src/index.tsx"], "folders": ["src/components"], "packageJson": {"dependencies": ["react"], "devDependencies": ["react-scripts"]}},
    "Next.js": {"files": ["next.config.js", "next.config.ts", "pages/_app.js", "pages/_app.tsx"], "packageJson": {"dependencies": ["next"]}},
    "Vue.js": {"files": ["src/App.vue", "src/main.js", "src/main.ts", "vue.config.js"], "packageJson": {"dependencies": ["vue"]}},
    "Nuxt.js": {"files": ["nuxt.config.js", "nuxt.config.ts"], "packageJson": {"dependencies": ["nuxt"]}},
    "Angular": {"files": ["angular.json", "src/main.ts", "src/app/app.module.ts"], "packageJson": {"dependencies": ["@angular/core"]}},
    "Svelte": {"files": ["svelte.config.js", "src/main.svelte", "src/App.svelte"], "packageJson": {"dependencies": ["svelte"]}},
    "SolidJS": {"packageJson": {"dependencies": ["solid-js"]}},
    "Astro": {"files": ["astro.config.mjs"], "packageJson": {"dependencies": ["astro"]}},
    "Remix": {"packageJson": {"dependencies": ["@remix-run/react"]}},
    "React Native": {"files": ["metro.config.js", "App.js", "App.tsx", "index.js", "index.tsx"], "packageJson": {"dependencies": ["react-native"]}},
    "Flutter": {"files": ["pubspec.yaml", "lib/main.dart", "android/app/src/main/AndroidManifest.xml"]},
    "Ionic": {"files": ["ionic.config.json"], "packageJson": {"dependencies": ["@ionic/core"]}},
    "NativeScript": {"packageJson": {"dependencies": ["nativescript"]}},
    "Cordova": {"files": ["config.xml"], "packageJson": {"dependencies": ["cordova"]}},
    "Capacitor": {"files": ["capacitor.config.json"], "packageJson": {"dependencies": ["@capacitor/core"]}},
    "Xamarin": {"files": [".sln", ".csproj"]},
    "Express.js": {"files": ["app.js", "server.js", "index.js", "express.js"], "packageJson": {"dependencies": ["express"]}},
    "NestJS": {"files": ["src/main.ts"], "packageJson": {"dependencies": ["@nestjs/core"]}},
    "Fastify": {"packageJson": {"dependencies": ["fastify"]}},
    "Koa": {"packageJson": {"dependencies": ["koa"]}},
    "Django": {"files": ["manage.py", "requirements.txt", "settings.py"], "folders": ["templates", "static"]},
    "Flask": {"files": ["app.py", "requirements.txt", "wsgi.py"], "contents": [{"file": "app.py", "keywords": ["Flask", "@app.route"]}]},
    "FastAPI": {"files": ["main.py", "requirements.txt"], "contents": [{"file": "main.py", "keywords": ["FastAPI"]}]},
    "Ruby on Rails": {"files": ["Gemfile", "config.ru", "Rakefile"], "contents": [{"file": "Gemfile", "keywords": ["rails"]}]},
    "Sinatra": {"files": ["app.rb", "config.ru"], "contents": [{"file": "app.rb", "keywords": ["Sinatra"]}]},
    "Laravel": {"files": ["artisan", "server.php", "composer.json"], "packageJson": {"dependencies": ["laravel-mix"]}},
    "Symfony": {"files": ["composer.json", "symfony.lock"], "contents": [{"file": "composer.json", "keywords": ["symfony/framework-bundle"]}]},
    "Phoenix": {"files": ["mix.exs"], "contents": [{"file": "mix.exs", "keywords": ["phoenix"]}]},
    "ASP.NET": {"files": [".csproj", "Program.cs", "Startup.cs"]},
    "Spring Boot": {"files": ["pom.xml", "build.gradle", "src/main/resources/application.properties"]},
    "Micronaut": {"files": ["build.gradle", "src/main/resources/application.yml"]},
    "Quarkus": {"files": ["pom.xml", "src/main/resources/application.properties"]},
    "JavaScript": {"files": ["package.json", "index.js"]},
    "TypeScript": {"files": ["tsconfig.json", "tsconfig.node.json"], "packageJson": {"devDependencies": ["typescript"]}},
    "Python": {"files": ["requirements.txt", "pyproject.toml", "setup.py", "Pipfile"]},
    "Java": {"files": ["pom.xml", "build.gradle", "src/main/java"]},
    "Kotlin": {"files": ["build.gradle.kts", "src/main/kotlin"]},
    "Go": {"files": ["go.mod", "go.sum", "main.go", "*.go"], "contents": [{"file": "go.mod", "keywords": ["module"]}]},
    "Rust": {"files": ["Cargo.toml", "Cargo.lock", "src/main.rs"]},
    "C": {"files": ["makefile", "*.c", "*.h"]},
    "C++": {"files": ["makefile", "*.cpp", "*.hpp"]},
    "C#": {"files": [".csproj", "Program.cs", "*.cs"]},
    "PHP": {"files": ["composer.json", "composer.lock", "index.php"]},
    "Ruby": {"files": ["Gemfile", "Gemfile.lock", "config.ru", "*.rb"]},
    "Swift": {"files": ["Package.swift", "*.swift"]},
    "Objective-C": {"files": ["*.m", "*.h"]},
    "Dart": {"files": ["pubspec.yaml", "*.dart"]},
    "Elixir": {"files": ["mix.exs", "mix.lock"]},
    "Scala": {"files": ["build.sbt", "project/build.properties"]},
    "Clojure": {"files": ["project.clj", "deps.edn"]},
    "Haskell": {"files": ["stack.yaml", "*.hs"]},
    "Perl": {"files": ["Makefile.PL", "*.pl"]},
    "R": {"files": ["DESCRIPTION", "*.R"]},
    "Julia": {"files": ["Project.toml", "*.jl"]},
    "Lua": {"files": ["*.lua"]},
    "Erlang": {"files": ["rebar.config", "*.erl"]},
    "OCaml": {"files": ["dune", "*.ml"]},
    "F#": {"files": ["*.fs", "*.fsx"]},
    "Groovy": {"files": ["build.gradle", "*.groovy"]},
    "D": {"files": ["dub.json", "*.d"]},
    "Nim": {"files": ["*.nim"]},
    "Zig": {"files": ["build.zig", "*.zig"]},
    "V": {"files": ["v.mod", "*.v"]},
    "Docker": {"files": ["Dockerfile", "docker-compose.yml"]},
    "Kubernetes": {"folders": ["k8s", "kubernetes"], "files": ["deployment.yaml", "service.yaml"]},
    "Terraform": {"files": ["main.tf", "variables.tf", "terraform.tfvars"]},
    "Serverless": {"files": ["serverless.yml", "serverless.js"]},
    "Ansible": {"folders": ["roles"], "files": ["playbook.yml"]},
    "Pulumi": {"files": ["Pulumi.yaml", "Pulumi.dev.yaml"]},
    "Bazel": {"files": ["WORKSPACE", "BUILD"]},
    "Make": {"files": ["Makefile", "makefile"]},
    "CMake": {"files": ["CMakeLists.txt"]},
    "Gradle": {"files": ["build.gradle", "settings.gradle"]},
    "Maven": {"files": ["pom.xml"]},
    "Webpack": {"files": ["webpack.config.js", "webpack.config.ts"], "packageJson": {"devDependencies": ["webpack"]}},
    "Vite": {"files": ["vite.config.js", "vite.config.ts"], "packageJson": {"devDependencies": ["vite"]}},
    "Rollup": {"files": ["rollup.config.js"], "packageJson": {"devDependencies": ["rollup"]}},
    "Parcel": {"files": [".parcelrc"], "packageJson": {"devDependencies": ["parcel"]}},
    "ESLint": {"files": [".eslintrc.js", ".eslintrc.json"], "packageJson": {"devDependencies": ["eslint"]}},
    "Prettier": {"files": [".prettierrc", ".prettierrc.js"], "packageJson": {"devDependencies": ["prettier"]}},
    "Babel": {"files": ["babel.config.js", ".babelrc"], "packageJson": {"devDependencies": ["@babel/core"]}},
    "Electron": {"files": ["electron-builder.json", "main.js"], "packageJson": {"dependencies": ["electron"]}},
    "Tauri": {"files": ["src-tauri/tauri.conf.json"], "packageJson": {"dependencies": ["@tauri-apps/cli"]}},
    "Qt": {"files": ["*.pro", "*.qml"]},
    "GTK": {"files": ["Makefile.am", "configure.ac"]},
    "WxWidgets": {"files": ["*.wxcp"]},
    "Jest": {"files": ["jest.config.js"], "packageJson": {"devDependencies": ["jest"]}},
    "Mocha": {"files": ["mocha.opts"], "packageJson": {"devDependencies": ["mocha"]}},
    "Jasmine": {"packageJson": {"devDependencies": ["jasmine"]}},
    "Cypress": {"folders": ["cypress"], "packageJson": {"devDependencies": ["cypress"]}},
    "Playwright": {"folders": ["tests"], "packageJson": {"devDependencies": ["@playwright/test"]}},
    "Puppeteer": {"packageJson": {"devDependencies": ["puppeteer"]}},
    "Selenium": {"files": ["requirements.txt"], "contents": [{"file": "requirements.txt", "keywords": ["selenium"]}]},
    "Vitest": {"packageJson": {"devDependencies": ["vitest"]}},
    "Gatsby": {"files": ["gatsby-config.js"], "packageJson": {"dependencies": ["gatsby"]}},
    "Hugo": {"files": ["config.toml", "config.yaml"]},
    "Jekyll": {"files": ["_config.yml"]},
    "Eleventy": {"files": [".eleventy.js"], "packageJson": {"dependencies": ["@11ty/eleventy"]}},
    "Docusaurus": {"files": ["docusaurus.config.js"], "packageJson": {"dependencies": ["@docusaurus/core"]}},
    "WordPress": {"files": ["wp-config.php", "wp-content"]},
    "Strapi": {"files": ["config/functions/bootstrap.js"], "packageJson": {"dependencies": ["strapi"]}},
    "Ghost": {"files": ["config.production.json"]},
    "Directus": {"packageJson": {"dependencies": ["directus"]}},
    "Sanity": {"files": ["sanity.json"], "packageJson": {"dependencies": ["@sanity/cli"]}},
    "PostgreSQL": {"files": ["init.sql", "*.pgsql"]},
    "MySQL": {"files": ["*.sql"]},
    "MongoDB": {"files": ["mongod.conf"]},
    "SQLite": {"files": ["*.sqlite", "*.db"]},
    "Redis": {"files": ["redis.conf"]},
    "Firebase": {"files": ["firebase.json"], "packageJson": {"dependencies": ["firebase"]}},
    "Supabase": {"files": ["supabase/config.toml"], "packageJson": {"dependencies": ["@supabase/supabase-js"]}},
    "GraphQL": {"files": ["schema.graphql"], "packageJson": {"dependencies": ["graphql"]}},
    "Apollo": {"packageJson": {"dependencies": ["@apollo/client"]}},
    "Hasura": {"files": ["metadata"]},
    "Wasm": {"files": ["*.wasm"]},
    "Emscripten": {"files": ["emcc"]},
    "TensorFlow": {"files": ["requirements.txt"], "contents": [{"file": "requirements.txt", "keywords": ["tensorflow"]}]},
    "PyTorch": {"files": ["requirements.txt"], "contents": [{"file": "requirements.txt", "keywords": ["torch"]}]},
    "Keras": {"files": ["requirements.txt"], "contents": [{"file": "requirements.txt", "keywords": ["keras"]}]},
    "Assembly": {"files": ["*.asm", "*.s"]},
    "Visual Basic": {"files": ["*.vb", "*.vbs"]},
    "CUDA": {"files": ["*.cu", "*.cuh"]},
    "Fortran": {"files": ["*.f90", "*.f95", "*.f03", "*.f"]},
    "COBOL": {"files": ["*.cbl", "*.cob", "*.cpy"]},
    "Lisp": {"files": ["*.lisp", "*.lsp"]},
    "Scheme": {"files": ["*.scm", "*.ss"]},
    "Racket": {"files": ["*.rkt"]},
    "Prolog": {"files": ["*.pl", "*.pro"]},
    "Smalltalk": {"files": ["*.st"]},
    "Ada": {"files": ["*.adb", "*.ads"]},
    "Pascal": {"files": ["*.pas", "*.pp"]},
    "Solidity": {"files": ["*.sol"], "packageJson": {"devDependencies": ["hardhat", "truffle"]}},
    "Vyper": {"files": ["*.vy"]},
    "Hack": {"files": ["*.hh", ".hhconfig"]},
    "Haxe": {"files": ["*.hx", "build.hxml"]},
    "Crystal": {"files": ["shard.yml", "*.cr"]},
    "Meteor": {"files": [".meteor"], "packageJson": {"dependencies": ["meteor-node-stubs"]}},
    "Aurelia": {"packageJson": {"dependencies": ["aurelia-framework"]}},
    "Mithril": {"packageJson": {"dependencies": ["mithril"]}},
    "Alpine.js": {"files": ["index.html"], "contents": [{"file": "index.html", "keywords": ["alpinejs"]}]},
    "Lit": {"packageJson": {"dependencies": ["lit", "lit-element"]}},
    "Stencil": {"files": ["stencil.config.ts"], "packageJson": {"dependencies": ["@stencil/core"]}},
    "Stimulus": {"packageJson": {"dependencies": ["@hotwired/stimulus"]}},
    "Turbo": {"packageJson": {"dependencies": ["@hotwired/turbo"]}},
    "Ember.js": {"files": ["ember-cli-build.js"], "packageJson": {"dependencies": ["ember-cli"]}},
    "Backbone.js": {"packageJson": {"dependencies": ["backbone"]}},
    "Knockout": {"packageJson": {"dependencies": ["knockout"]}},
    "Polymer": {"files": ["polymer.json"], "packageJson": {"dependencies": ["@polymer/polymer"]}},
    "Nx": {"files": ["nx.json"]},
    "Lerna": {"files": ["lerna.json"]},
    "Turborepo": {"files": ["turbo.json"]},
    "Rush": {"files": ["rush.json"]},
    "Yarn Workspaces": {"packageJson": {"scripts": ["workspaces"]}},
    "PNPM Workspaces": {"files": ["pnpm-workspace.yaml"]},
}


def _read_file_safe(path):
    try:
        with open(path, "r", encoding="utf-8", errors="ignore") as f:
            return f.read()
    except Exception:
        return ""


def _path_exists(p):
    return os.path.exists(p)


def _is_dir(p):
    return os.path.isdir(p)


def _list_dir_safe(p):
    try:
        return os.listdir(p)
    except Exception:
        return []


def get_category(type_name):
    frontend = [
        "React", "Vue.js", "Angular", "Svelte", "Next.js", "Nuxt.js", "SolidJS", "Astro", "Remix", "Gatsby", "Hugo",
        "Jekyll", "Eleventy", "Docusaurus", "Qwik", "Analog", "SvelteKit", "Fresh", "Meteor", "Aurelia", "Mithril",
        "Alpine.js", "Lit", "Stencil", "Stimulus", "Turbo", "Ember.js", "Backbone.js", "Knockout", "Polymer"
    ]
    backend = [
        "Express.js", "NestJS", "Fastify", "Koa", "Hono", "tRPC", "Django", "Flask", "FastAPI", "Ruby on Rails",
        "Sinatra", "Laravel", "Symfony", "Phoenix", "ASP.NET", "Spring Boot", "Micronaut", "Quarkus", "Go", "Rust",
        "Prisma", "DrizzleORM", "TypeORM", "Sequelize", "PocketBase", "Assembly", "Visual Basic", "CUDA", "Fortran",
        "COBOL", "Lisp", "Scheme", "Racket", "Prolog", "Smalltalk", "Ada", "Pascal", "Solidity", "Vyper", "Hack",
        "Haxe", "Julia", "Crystal"
    ]
    mobile = ["React Native", "Flutter", "Ionic", "NativeScript", "Cordova", "Capacitor", "Xamarin"]
    desktop = ["Electron", "Tauri", "Qt", "GTK", "WxWidgets"]
    devops = [
        "Docker", "Kubernetes", "Terraform", "Serverless", "Ansible", "Pulumi", "Bazel", "Nx", "Lerna",
        "Turborepo", "Rush", "Yarn Workspaces", "PNPM Workspaces", "Make", "CMake", "Gradle", "Maven"
    ]
    if any(t in type_name for t in frontend):
        return "frontend"
    if any(t in type_name for t in backend):
        return "backend"
    if any(t in type_name for t in mobile):
        return "mobile"
    if any(t in type_name for t in desktop):
        return "desktop"
    if any(t in type_name for t in devops):
        return "devops"
    return "other"


def detect_project_type(project_dir):
    detected_types = []

    go_mod_path = os.path.join(project_dir, "go.mod")
    try:
        go_files = [f for f in os.listdir(project_dir) if f.endswith(".go")]
    except Exception:
        go_files = []

    has_go_files = len(go_files) > 0
    has_go_mod = _path_exists(go_mod_path)

    if has_go_mod or has_go_files:
        go_type = "Go"
        confidence = 100 if has_go_mod else 80

        go_mod_content = _read_file_safe(go_mod_path) if has_go_mod else ""
        main_go_content = _read_file_safe(os.path.join(project_dir, "main.go"))

        frameworks = {
            "github.com/gin-gonic/gin": "Gin",
            "github.com/gorilla/mux": "Gorilla Mux",
            "github.com/labstack/echo": "Echo",
            "github.com/gofiber/fiber": "Fiber",
            "github.com/go-chi/chi": "Chi",
            "github.com/graphql-go/graphql": "GraphQL",
            "github.com/grpc/grpc-go": "gRPC",
            "gorm.io/gorm": "GORM",
        }

        for pkg, framework in frameworks.items():
            if pkg in go_mod_content or pkg in main_go_content:
                go_type = f"Go {framework}"
                confidence += 10

        detected_types.append({"type": go_type, "category": "backend", "confidence": min(confidence, 100)})

    package_json = {}
    package_json_path = os.path.join(project_dir, "package.json")
    if _path_exists(package_json_path):
        try:
            with open(package_json_path, "r", encoding="utf-8") as f:
                package_json = json.load(f)
        except Exception:
            pass

    dependencies = package_json.get("dependencies", {})
    dev_dependencies = package_json.get("devDependencies", {})
    all_deps = {**dependencies, **dev_dependencies}

    if package_json:
        is_react_app = any(k in all_deps for k in ["react", "next", "gatsby", "remix"])
        is_backend = (
            "express" in all_deps
            or "@nestjs/core" in all_deps
            or "fastify" in all_deps
            or "koa" in all_deps
            or "hono" in all_deps
            or "@trpc/server" in all_deps
            or package_json.get("type") == "module"
            or _path_exists(os.path.join(project_dir, "server.js"))
            or _path_exists(os.path.join(project_dir, "app.js"))
            or _path_exists(os.path.join(project_dir, "index.js"))
            or _is_dir(os.path.join(project_dir, "src", "server"))
            or _is_dir(os.path.join(project_dir, "src", "api"))
        )

        if not is_react_app and is_backend:
            node_type = "TypeScript Node.js" if "typescript" in all_deps else "Node.js"

            if "express" in all_deps:
                node_type += " Express"
            elif "@nestjs/core" in all_deps:
                node_type += " NestJS"
            elif "fastify" in all_deps:
                node_type += " Fastify"
            elif "koa" in all_deps:
                node_type += " Koa"
            elif "hono" in all_deps:
                node_type += " Hono"
            elif "@trpc/server" in all_deps:
                node_type += " tRPC"

            if "@prisma/client" in all_deps:
                node_type += " + Prisma"
            if "typeorm" in all_deps:
                node_type += " + TypeORM"
            if "sequelize" in all_deps:
                node_type += " + Sequelize"
            if "drizzle-orm" in all_deps:
                node_type += " + DrizzleORM"
            if "mongoose" in all_deps:
                node_type += " + Mongoose"

            detected_types.append({"type": node_type, "category": "backend", "confidence": 95})

    for type_name, pattern in detection_patterns.items():
        if type_name == "Go" and any(t["type"] == "Go" for t in detected_types):
            continue

        confidence = 0

        if "files" in pattern:
            for file_pattern in pattern["files"]:
                if "*" in file_pattern:
                    if glob.glob(os.path.join(project_dir, file_pattern)):
                        confidence += 30
                        break
                elif _path_exists(os.path.join(project_dir, file_pattern)):
                    confidence += 30
                    break

        if "folders" in pattern:
            for folder in pattern["folders"]:
                if _is_dir(os.path.join(project_dir, folder)):
                    confidence += 20
                    break

        if "contents" in pattern:
            for content_check in pattern["contents"]:
                file_path = os.path.join(project_dir, content_check["file"])
                if _path_exists(file_path):
                    content = _read_file_safe(file_path)
                    if any(kw in content for kw in content_check["keywords"]):
                        confidence += 25

        if "packageJson" in pattern and package_json:
            pj = pattern["packageJson"]
            if "dependencies" in pj:
                if any(dep in dependencies for dep in pj["dependencies"]):
                    confidence += 25
            if "devDependencies" in pj:
                if any(dep in dev_dependencies for dep in pj["devDependencies"]):
                    confidence += 15
            if "scripts" in pj:
                scripts = package_json.get("scripts", {})
                if any(s in scripts for s in pj["scripts"]):
                    confidence += 10

        if confidence > 0:
            detected_types.append({"type": type_name, "category": get_category(type_name), "confidence": confidence})

    requirements_path = os.path.join(project_dir, "requirements.txt")
    pipfile_path = os.path.join(project_dir, "Pipfile")
    poetry_path = os.path.join(project_dir, "pyproject.toml")
    conda_path = os.path.join(project_dir, "environment.yml")
    try:
        python_files = [f for f in os.listdir(project_dir) if f.endswith(".py")]
    except Exception:
        python_files = []

    is_python_project = (
        len(python_files) > 0
        or _path_exists(requirements_path)
        or _path_exists(pipfile_path)
        or _path_exists(poetry_path)
        or _path_exists(conda_path)
    )

    if is_python_project:
        python_type = "Python"
        py_confidence = 90
        py_frameworks = []

        all_py_deps = (
            _read_file_safe(requirements_path)
            + _read_file_safe(pipfile_path)
            + _read_file_safe(poetry_path)
            + _read_file_safe(conda_path)
            + _read_file_safe(os.path.join(project_dir, "main.py"))
            + _read_file_safe(os.path.join(project_dir, "app.py"))
        )

        framework_patterns = {
            "django": {"name": "Django", "score": 15, "files": ["manage.py", "wsgi.py", "asgi.py"]},
            "fastapi": {"name": "FastAPI", "score": 15, "imports": ["from fastapi import"]},
            "flask": {"name": "Flask", "score": 15, "imports": ["from flask import"]},
            "tornado": {"name": "Tornado", "score": 15, "imports": ["import tornado"]},
            "pyramid": {"name": "Pyramid", "score": 15, "imports": ["from pyramid.config import"]},
            "aiohttp": {"name": "AIOHTTP", "score": 15, "imports": ["import aiohttp"]},
            "sanic": {"name": "Sanic", "score": 15, "imports": ["from sanic import"]},
            "dash": {"name": "Dash", "score": 10, "imports": ["import dash"]},
            "streamlit": {"name": "Streamlit", "score": 10, "imports": ["import streamlit"]},
        }

        for key, framework in framework_patterns.items():
            if key in all_py_deps.lower():
                py_frameworks.append(framework["name"])
                py_confidence += framework["score"]

                for f in framework.get("files", []):
                    if _path_exists(os.path.join(project_dir, f)):
                        py_confidence += 5

                for imp in framework.get("imports", []):
                    if imp in all_py_deps:
                        py_confidence += 5

        libraries = {
            "sqlalchemy": "SQLAlchemy",
            "peewee": "Peewee",
            "tortoise": "TortoiseORM",
            "mongoengine": "MongoEngine",
            "pydantic": "Pydantic",
            "celery": "Celery",
            "pytest": "PyTest",
            "numpy": "NumPy",
            "pandas": "Pandas",
            "tensorflow": "TensorFlow",
            "torch": "PyTorch",
        }

        for lib, name in libraries.items():
            if lib in all_py_deps.lower():
                py_frameworks.append(name)
                py_confidence += 5

        if py_frameworks:
            python_type = f"Python {' + '.join(py_frameworks)}"

        web_frameworks = {"Django", "FastAPI", "Flask", "Tornado", "Pyramid", "AIOHTTP", "Sanic"}
        category = "backend" if any(f in web_frameworks for f in py_frameworks) else "other"

        detected_types.append({"type": python_type, "category": category, "confidence": min(py_confidence, 100)})

    gemfile_path = os.path.join(project_dir, "Gemfile")
    try:
        ruby_files = [f for f in os.listdir(project_dir) if f.endswith(".rb")]
    except Exception:
        ruby_files = []

    if _path_exists(gemfile_path) or len(ruby_files) > 0:
        ruby_type = "Ruby"
        rb_confidence = 90
        rb_frameworks = []

        all_gems = (
            _read_file_safe(gemfile_path)
            + _read_file_safe(os.path.join(project_dir, "Gemfile.lock"))
            + _read_file_safe(os.path.join(project_dir, "config.ru"))
        )

        gem_patterns = {
            "rails": {
                "name": "Rails",
                "score": 20,
                "files": ["config/routes.rb", "config/application.rb", "app/controllers/application_controller.rb"],
            },
            "sinatra": {
                "name": "Sinatra",
                "score": 15,
                "requires": ['require "sinatra"', "require 'sinatra'"],
            },
            "hanami": {
                "name": "Hanami",
                "score": 15,
                "files": ["config/environment.rb", "apps/web/application.rb"],
            },
            "grape": {
                "name": "Grape",
                "score": 15,
                "requires": ['require "grape"', "require 'grape'"],
            },
            "roda": {
                "name": "Roda",
                "score": 15,
                "requires": ['require "roda"', "require 'roda'"],
            },
        }

        for key, gem_pattern in gem_patterns.items():
            if key in all_gems.lower():
                rb_frameworks.append(gem_pattern["name"])
                rb_confidence += gem_pattern["score"]

                for f in gem_pattern.get("files", []):
                    if _path_exists(os.path.join(project_dir, f)):
                        rb_confidence += 5

                for req in gem_pattern.get("requires", []):
                    if req in all_gems:
                        rb_confidence += 5

        gems = {
            "devise": "Devise",
            "activerecord": "ActiveRecord",
            "sequel": "Sequel",
            "mongoid": "Mongoid",
            "sidekiq": "Sidekiq",
            "rspec": "RSpec",
            "minitest": "Minitest",
            "capybara": "Capybara",
            "webpacker": "Webpacker",
            "stimulus": "Stimulus",
            "turbo-rails": "Turbo",
        }

        for gem, name in gems.items():
            if gem in all_gems.lower():
                rb_frameworks.append(name)
                rb_confidence += 5

        if rb_frameworks:
            ruby_type = f"Ruby {' + '.join(rb_frameworks)}"

        detected_types.append({"type": ruby_type, "category": "backend", "confidence": min(rb_confidence, 100)})

    if package_json and "react" in all_deps:
        react_type = "TypeScript React" if ("@types/react" in all_deps or "typescript" in all_deps) else "React"

        if "next" in all_deps:
            react_type += " Next.js"
        elif "gatsby" in all_deps:
            react_type += " Gatsby"
        elif "remix" in all_deps:
            react_type += " Remix"

        if "@mui/material" in all_deps:
            react_type += " Material-UI"
        elif "@chakra-ui/react" in all_deps:
            react_type += " Chakra UI"
        elif "antd" in all_deps:
            react_type += " Ant Design"
        elif "@tailwindcss/react" in all_deps:
            react_type += " Tailwind"

        detected_types.append({"type": react_type, "category": "frontend", "confidence": 95})

    has_client_dir = _is_dir(os.path.join(project_dir, "client"))
    has_server_dir = _is_dir(os.path.join(project_dir, "server"))
    has_apps_dir = _is_dir(os.path.join(project_dir, "apps"))
    has_packages_dir = _is_dir(os.path.join(project_dir, "packages"))

    if (has_client_dir and has_server_dir) or (has_apps_dir and has_packages_dir):
        client_types = []
        server_types = []
        is_monorepo = False

        if has_client_dir and has_server_dir:
            is_monorepo = True
            client_result = detect_project_type(os.path.join(project_dir, "client"))
            server_result = detect_project_type(os.path.join(project_dir, "server"))
            client_types.append(client_result)
            server_types.append(server_result)

        if has_apps_dir:
            is_monorepo = True
            for app in _list_dir_safe(os.path.join(project_dir, "apps")):
                app_type = detect_project_type(os.path.join(project_dir, "apps", app))
                app_lower = app_type.lower()
                if any(k in app_lower for k in ["react", "vue", "angular", "front"]):
                    client_types.append(app_type)
                else:
                    server_types.append(app_type)

        if has_packages_dir:
            is_monorepo = True
            for pkg in _list_dir_safe(os.path.join(project_dir, "packages")):
                pkg_type = detect_project_type(os.path.join(project_dir, "packages", pkg))
                if pkg_type != "Unknown":
                    pkg_lower = pkg_type.lower()
                    if any(k in pkg_lower for k in ["react", "vue", "angular", "front"]):
                        client_types.append(pkg_type)
                    else:
                        server_types.append(pkg_type)

        if is_monorepo:
            unique_client = list(dict.fromkeys(t for t in client_types if t != "Unknown"))
            unique_server = list(dict.fromkeys(t for t in server_types if t != "Unknown"))

            client_str = f"Client: {' + '.join(unique_client)}" if unique_client else ""
            server_str = f"Server: {' + '.join(unique_server)}" if unique_server else ""

            parts = [p for p in [client_str, server_str] if p]
            return f"Monorepo [{' | '.join(parts)}]"

    if detected_types:
        detected_types.sort(key=lambda x: x["confidence"], reverse=True)
        return detected_types[0]["type"]

    return "Unknown"



import os
import sys
import time
import subprocess
import gzip
import base64
import requests
from rich.live import Live
from rich.spinner import Spinner
from rich.console import Console
from pathlib import Path
import pathspec
import platform
import datetime

console = Console()
readme_backup = None
current_readme_path = ""

PACKAGE_NAME = "dokugen"
PYPI_URL = f"https://pypi.org/pypi/{PACKAGE_NAME}/json"


def create_spinner(text):
    return Live(Spinner("dots", text=text), refresh_per_second=10)


def sleep(ms):
    time.sleep(ms / 1000.0)


def get_installed_version():
    """Get the currently installed version of dokugen."""
    try:
        from importlib.metadata import version as get_version
        return get_version(PACKAGE_NAME)
    except Exception:
        return None


def check_and_update():
    try:
        current_version = get_installed_version()
        if not current_version:
            return

        response = requests.get(PYPI_URL, timeout=3)
        if response.status_code != 200:
            return

        latest_version = response.json()["info"]["version"]

        if latest_version != current_version:
            console.print(f"\n[cyan]New version available: {latest_version} (current: {current_version})[/cyan]")

            with create_spinner(f"Updating {PACKAGE_NAME}..."):
                try:
                    subprocess.run(
                        [sys.executable, "-m", "uv", "pip", "install", "--upgrade", f"{PACKAGE_NAME}=={latest_version}"],
                        capture_output=True, timeout=60, check=True
                    )
                except (subprocess.CalledProcessError, FileNotFoundError):
                    try:
                        subprocess.run(
                            [sys.executable, "-m", "pip", "install", "--upgrade", f"{PACKAGE_NAME}=={latest_version}"],
                            capture_output=True, timeout=60, check=True
                        )
                    except Exception:
                        console.print(f"[yellow]Auto-update failed. Please run: pip install --upgrade {PACKAGE_NAME}[/yellow]")
                        return

            console.print(f"[green]Successfully updated to v{latest_version}![/green]")
            console.print("[yellow]Please re-run your command to use the new version.\n[/yellow]")
            sys.exit(0)
    except Exception:
        return


def get_user_info():
    git_name = ""
    git_email = ""
    try:
        git_name = subprocess.check_output(["git", "config", "--get", "user.name"], encoding="utf-8").strip()
        git_email = subprocess.check_output(["git", "config", "--get", "user.email"], encoding="utf-8").strip()
    except subprocess.CalledProcessError:
        console.print("[yellow]Git User Info not found. Using Defaults......[/yellow]")

    os_info = {
        "platform": platform.system() or "unknown",
        "arch": platform.machine() or "unknown",
        "release": platform.release() or "unknown",
    }

    if git_name and git_email:
        return {"username": git_name, "email": git_email, "osInfo": os_info}

    try:
        username = os.getlogin()
    except Exception:
        username = "Unknown"

    return {
        "username": username,
        "email": os.environ.get("USER", ""),
        "osInfo": os_info
    }


def check_internet_connection():
    try:
        requests.get("https://www.google.com", timeout=5)
        return True
    except Exception:
        return False


def compress_data(data):
    compressed = gzip.compress(data.encode('utf-8'))
    return base64.b64encode(compressed).decode('utf-8')


def backup_readme(readme_path):
    global readme_backup, current_readme_path
    if os.path.exists(readme_path):
        current_readme_path = readme_path
        with open(readme_path, 'r', encoding='utf-8', errors='ignore') as f:
            readme_backup = f.read()
        console.print(f"[green][{datetime.datetime.now().isoformat()}] Current README backed up in memory[/green]")


def restore_readme():
    global readme_backup, current_readme_path
    if readme_backup and current_readme_path:
        try:
            with open(current_readme_path, "w", encoding="utf-8") as f:
                f.write(readme_backup)
            console.print("[green]Original README content restored successfully[/green]")
            return readme_backup
        except Exception as e:
            console.print(f"[red]Failed to restore README: {e}[/red]")
            return None
        finally:
            readme_backup = None
            current_readme_path = ""
    else:
        console.print("[yellow]No README backup available to restore[/yellow]")
        return None


def get_git_repo_url():
    try:
        url = subprocess.check_output(["git", "config", "--get", "remote.origin.url"], encoding="utf-8").strip()
        return url if url else None
    except Exception:
        return None


def matches_ignore_pattern(filename, pattern):
    if pattern.startswith("*."):
        ext = pattern[1:]
        return filename.endswith(ext)
    return filename == pattern


def scan_files(root_dir):
    ignore_dirs = {
        "node_modules", "bower_components", "jspm_packages", "web_modules",
        "dist", "build", "out", "target", "bin", "obj", "lib", "release", "debug",
        "artifacts", "generated", "temp", "tmp", "cache", ".cache", ".temp",
        ".next", ".nuxt", ".svelte-kit", ".vercel", ".serverless", ".expo", ".output",
        "dist-electron", "release-builds", ".parcel-cache",
        "android", "ios", "windows", "linux", "macos", "web",
        ".dart_tool", ".pub-cache", ".pub", "Pods", ".bundle",
        "venv", ".venv", "env", ".env", "virtualenv", "envs", "__pycache__",
        ".pytest_cache", ".mypy_cache", ".tox", "htmlcov", "site-packages",
        "vendor", "var", "storage",
        ".gradle", ".mvn", ".idea",
        "tests", "_tests_", "_test_", "__tests__", "coverage", "test", "spec",
        "cypress", "e2e", "reports",
        ".git", ".svn", ".hg", ".vscode", ".vs", ".history", ".github", ".gitlab",
        "public", "static", "assets", "images", "media", "uploads", "fonts", "icons",
        "migrations", "data", "db", "database", "logs", "log", "dump", "backups",
        "docs", "javadoc", "tools", "scripts", "config", "settings",
        "cmake-build-debug", "packages", "plugins", "examples", "samples"
    }

    ignore_files = {
        "*.exe", "*.dll", "*.so", "*.dylib", "*.bin", "*.iso", "*.img", "*.dmg",
        "*.zip", "*.tar", "*.gz", "*.rar", "*.7z", "*.bz2", "*.xz",
        "*.mp4", "*.mkv", "*.avi", "*.mov", "*.wmv", "*.flv", "*.webm",
        "*.mp3", "*.wav", "*.flac", "*.aac", "*.ogg", "*.wma",
        "*.jpg", "*.jpeg", "*.png", "*.gif", "*.bmp", "*.ico", "*.svg", "*.webp", "*.tiff",
        "*.pdf", "*.doc", "*.docx", "*.ppt", "*.pptx", "*.xls", "*.xlsx", "*.csv",
        "*.ttf", "*.otf", "*.woff", "*.woff2", "*.eot",
        "*.pyc", "*.pyo", "*.pyd",
        "*.class", "*.jar", "*.war", "*.ear",
        "*.o", "*.obj", "*.a", "*.lib",
        "*.lock", "package-lock.json", "yarn.lock", "pnpm-lock.yaml", "Gemfile.lock", "composer.lock", "mix.lock", "pubspec.lock", "Cargo.lock",
        "*.log", "*.tmp", "*.temp", "*.swp", "*.swo", "*.bak", "*.old", "*.orig",
        ".DS_Store", "Thumbs.db", "desktop.ini",
        ".env", ".env.local", ".env.development", ".env.test", ".env.production",
        "Dockerfile", "docker-compose.yml", "Makefile", "CMakeLists.txt",
        "LICENSE", "CHANGELOG.md", "CONTRIBUTING.md", "CODE_OF_CONDUCT.md", "README.md",
        ".gitignore", ".npmignore", ".dockerignore", ".eslintrc*", ".prettierrc*", "tsconfig.json",
        "*.min.js", "*.min.css", "*.map", "*.d.ts", "*.apk", "*.aab", "*.ipa", "*.hap"
    }

    gitignore_spec = None
    gitignore_path = os.path.join(root_dir, ".gitignore")
    if os.path.exists(gitignore_path):
        try:
            with open(gitignore_path, 'r') as f:
                gitignore_spec = pathspec.PathSpec.from_lines('gitwildmatch', f)
        except Exception:
            pass

    found_files = []

    for dirpath, dirnames, filenames in os.walk(root_dir):
        dirnames[:] = [d for d in dirnames if d not in ignore_dirs]

        rel_dir = os.path.relpath(dirpath, root_dir)
        if rel_dir == ".":
            rel_dir = ""

        if gitignore_spec and rel_dir and gitignore_spec.match_file(rel_dir):
            dirnames[:] = []
            continue

        for filename in filenames:
            should_ignore = False
            for pattern in ignore_files:
                if matches_ignore_pattern(filename, pattern):
                    should_ignore = True
                    break

            if should_ignore:
                continue

            rel_file_path = os.path.join(rel_dir, filename)

            if gitignore_spec and gitignore_spec.match_file(rel_file_path):
                continue

            found_files.append(rel_file_path)

    return found_files


def extract_full_code(project_files, project_dir):
    snippets = []

    file_groups = {}
    for f in project_files:
        d = os.path.dirname(f)
        if d not in file_groups:
            file_groups[d] = []
        file_groups[d].append(f)

    for d, files in file_groups.items():
        dir_snippets = []
        for file in files:
            file_path = os.path.join(project_dir, file)
            try:
                size_kb = os.path.getsize(file_path) / 1024
                with open(file_path, 'r', encoding='utf-8', errors='replace') as f:
                    content = f.read()

                ext = Path(file).suffix[1:] or "txt"
                dir_snippets.append(f"### {file}\n- **Path:** {file}\n- **Size:** {size_kb:.2f} KB\n```{ext}\n{content}\n```\n")
            except Exception as e:
                console.print(f"[red]Failed to read file: {file} - {e}[/red]")

        if dir_snippets:
            snippets.append(f"## {d}\n" + "".join(dir_snippets))

    return "".join(snippets) or "No code snippets available"



#!/usr/bin/env python3
import os
import sys
import json
import argparse
import requests
import questionary
from rich.console import Console
from dokugen.project_detect import detect_project_type
from dokugen import utils

console = Console()

DOKUGEN_BANNER = """
   ___   ____  __ ____  _____________  __
  / _ \\ / __ \\/ //_/ / / / ____/ ____/ / /
 / / / / / / / ,< / / / / / __/ __/ / /_ 
/ /_/ / /_/ / /| / /_/ / /_/ / /___/ / / 
\\____/\\____/_/ |_\\____/\\____/_____/_/ /  
                                   /_/   
"""

API_TIMEOUT = 300


def ask_yes_no(message):
    try:
        answer = questionary.select(
            message,
            choices=["Yes", "No"],
        ).ask()
        if answer is None:
            return "cancel"
        return "yes" if answer == "Yes" else "no"
    except KeyboardInterrupt:
        return "cancel"


def ask_for_gemini_key():
    has_key = questionary.select(
        "Do you have a Google Gemini API Key?",
        choices=[
            questionary.Choice("Yes", value="yes"),
            questionary.Choice("No (Use Dokugen's shared key)", value="no"),
        ]
    ).ask()

    if has_key is None:
        console.print("[yellow]Operation cancelled[/yellow]")
        sys.exit(0)

    if has_key == "yes":
        key = questionary.text(
            "Enter your Google Gemini API Key:",
            validate=lambda val: True if val else "Value is required!"
        ).ask()
        if key is None:
            console.print("[yellow]Operation cancelled[/yellow]")
            sys.exit(0)
        return key
    return None


def generate_readme_remote(project_type, project_files, project_dir, existing_readme=None, template_url=None, gemini_api_key=None):
    try:
        console.print("[blue]Analyzing project files...[/blue]")
        readme_path = os.path.join(project_dir, "README.md")

        include_setup = False
        include_contrib = False

        if not template_url:
            ans_setup = ask_yes_no("Do you want to include setup instructions in the README?")
            if ans_setup == "cancel":
                return None
            include_setup = (ans_setup == "yes")

            ans_contrib = ask_yes_no("Include contribution guidelines in README?")
            if ans_contrib == "cancel":
                return None
            include_contrib = (ans_contrib == "yes")

        full_code = utils.extract_full_code(project_files, project_dir)
        user_info = utils.get_user_info()
        repo_url = utils.get_git_repo_url()

        try:
            r = requests.get("https://dokugen-readme.vercel.app/api/get-server-url", timeout=10)
            backend_domain = r.json().get("domain")
        except Exception:
            backend_domain = "https://dokugen-readme.vercel.app"

        compressed_full_code = utils.compress_data(full_code)
        compressed_existing_readme = utils.compress_data(existing_readme) if existing_readme else None

        payload = {
            "projectType": project_type,
            "projectFiles": project_files,
            "fullCode": compressed_full_code,
            "userInfo": user_info,
            "options": {"includeSetup": include_setup, "includeContributionGuideLine": include_contrib},
            "existingReadme": compressed_existing_readme,
            "repoUrl": repo_url,
            "templateUrl": template_url,
            "compressed": True,
            "geminiApiKey": gemini_api_key,
        }

        spinner = utils.create_spinner("Generating README...")
        spinner.__enter__()

        try:
            response = requests.post(
                f"{backend_domain}/api/generate-readme",
                json=payload,
                stream=True,
                timeout=API_TIMEOUT,
            )
        finally:
            spinner.__exit__(None, None, None)

        if response.status_code != 200:
            console.print(f"[red]Error {response.status_code}: {response.text}[/red]")
            utils.restore_readme()
            return None

        with open(readme_path, "w", encoding="utf-8") as f:
            for line in response.iter_lines():
                if line:
                    decoded = line.decode("utf-8")
                    if decoded.startswith("data:"):
                        json_str = decoded.replace("data: ", "").strip()
                        try:
                            data = json.loads(json_str)
                            if "response" in data and isinstance(data["response"], str):
                                f.write(data["response"])
                                f.flush()
                        except Exception:
                            pass

        console.print("\n[green]README.md created successfully[/green]")
        utils.readme_backup = None
        return readme_path

    except Exception as e:
        console.print(f"[red]\n Error Generating Readme: {e}[/red]")
        utils.restore_readme()
        return None


def cmd_generate(args):
    utils.check_and_update()
    console.print(DOKUGEN_BANNER, style="#000080")

    project_dir = os.getcwd()
    readme_path = os.path.join(project_dir, "README.md")
    readme_exists = os.path.exists(readme_path)

    with utils.create_spinner("Checking internet..."):
        has_connection = utils.check_internet_connection()

    if not has_connection:
        username = utils.get_user_info().get("username", "Unknown")
        console.print(f"[red]Opps... {username} kindly check your device or pc internet connection and try again.[/red]")
        return

    if readme_exists:
        utils.backup_readme(readme_path)

    try:
        template_url = getattr(args, "template", None)
        if template_url and "github.com" not in template_url:
            console.print("[red]Invalid GitHub URL. Use format: https://github.com/user/repo/blob/main/README.md[/red]")
            sys.exit(1)

        project_type = detect_project_type(project_dir)

        with utils.create_spinner("Scanning project files...") as spinner:
            project_files = utils.scan_files(project_dir)

        console.print(f"[yellow]Found: {len(project_files)} files in the project[/yellow]")
        console.print(f"[blue]Detected project type: {project_type}[/blue]")

        gemini_key = ask_for_gemini_key()

        if template_url:
            existing_content = None
            if readme_exists and not getattr(args, "overwrite", True):
                with open(readme_path, "r", encoding="utf-8") as f:
                    existing_content = f.read()
            generate_readme_remote(project_type, project_files, project_dir, existing_content, template_url, gemini_key)
            console.print("[green]README.md generated from template![/green]")
            return

        if readme_exists:
            if not getattr(args, "overwrite", True):
                with open(readme_path, "r", encoding="utf-8") as f:
                    existing_content = f.read()
                generate_readme_remote(project_type, project_files, project_dir, existing_content, None, gemini_key)
            else:
                ans = ask_yes_no("README.md exists. Overwrite?")
                if ans == "yes":
                    generate_readme_remote(project_type, project_files, project_dir, None, None, gemini_key)
                elif ans == "no":
                    console.print("[yellow]README update skipped (user selected No)[/yellow]")
                    return
                elif ans == "cancel":
                    console.print("[yellow]README generation cancelled[/yellow]")
                    utils.restore_readme()
                    return
        else:
            generate_readme_remote(project_type, project_files, project_dir, None, None, gemini_key)

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        utils.restore_readme()
        sys.exit(1)


def cmd_update(args):
    utils.check_and_update()
    console.print(DOKUGEN_BANNER, style="#000080")

    project_dir = os.getcwd()
    readme_path = os.path.join(project_dir, "README.md")

    if not os.path.exists(readme_path):
        console.print("[red]No README.md found. Use 'dokugen generate' to create one first.[/red]")
        sys.exit(1)

    with utils.create_spinner("Checking internet..."):
        has_connection = utils.check_internet_connection()

    if not has_connection:
        username = utils.get_user_info().get("username", "Unknown")
        console.print(f"[red]Opps... {username} kindly check your device or pc internet connection and try again.[/red]")
        return

    try:
        utils.backup_readme(readme_path)
        gemini_key = ask_for_gemini_key()

        template_url = getattr(args, "template", None)

        if template_url and "github.com" not in template_url:
            console.print("[red]Invalid GitHub URL. Use format: https://github.com/user/repo/blob/main/README.md[/red]")
            sys.exit(1)

        with open(readme_path, "r", encoding="utf-8") as f:
            existing_content = f.read()

        has_markers = "<!-- DOKUGEN:" in existing_content or "[![Readme was generated by Dokugen]" in existing_content

        if not has_markers:
            console.print("[yellow]This README doesn't appear to be generated by Dokugen.[/yellow]")
            ans = ask_yes_no("Do you want to regenerate the entire README?")

            if ans == "yes":
                project_type = detect_project_type(project_dir)
                with utils.create_spinner("Scanning project files..."):
                    project_files = utils.scan_files(project_dir)
                console.print(f"[yellow]Found: {len(project_files)} files in the project[/yellow]")

                generate_readme_remote(project_type, project_files, project_dir, None, template_url, gemini_key)
                console.print("[green]README.md regenerated successfully![/green]")
                return
            else:
                console.print("[yellow]Update cancelled.[/yellow]")
                utils.restore_readme()
                return

        console.print("[blue]Analyzing README structure...[/blue]")
        project_type = detect_project_type(project_dir)

        with utils.create_spinner("Scanning project files..."):
            project_files = utils.scan_files(project_dir)

        console.print(f"[yellow]Found: {len(project_files)} files in the project[/yellow]")
        console.print(f"[blue]Detected project type: {project_type}[/blue]")
        console.print("[blue]Updating auto-generated sections...[/blue]")

        generate_readme_remote(project_type, project_files, project_dir, existing_content, template_url, gemini_key)
        console.print("[green]README.md updated successfully! Custom sections preserved.[/green]")

    except Exception as e:
        console.print(f"[red]Error updating README: {e}[/red]")
        utils.restore_readme()
        sys.exit(1)


def cmd_compose_docker(args):
    console.print("testing coming soon.....")


def main():
    parser = argparse.ArgumentParser(
        prog="dokugen",
        description="Automatically generate high-quality README for your application",
    )
    parser.add_argument("--version", "-v", action="version", version="%(prog)s 3.12.1")

    subparsers = parser.add_subparsers(dest="command")

    gen_parser = subparsers.add_parser("generate", help="Scan project and generate README.md")
    gen_parser.add_argument(
        "--no-overwrite",
        dest="overwrite",
        action="store_false",
        default=True,
        help="Do not overwrite existing README.md, append new features instead",
    )
    gen_parser.add_argument(
        "--template",
        help="use a custom GitHub repo readme file as a template to generate a concise and strict readme for your project",
    )

    up_parser = subparsers.add_parser("update", help="Update auto-generated sections of README while preserving custom content")
    up_parser.add_argument(
        "--template",
        help="use a custom GitHub repo readme file as a template",
    )

    subparsers.add_parser("compose-docker", help="Generate dockerfiles or docker compose for multiple services of your project")

    if len(sys.argv) == 1:
        console.print(DOKUGEN_BANNER, style="#000080")
        console.print("[blue]Welcome to Dokugen - Automatic README Generator\n[/blue]")

        action = questionary.select(
            "What would you like to do?",
            choices=[
                questionary.Choice("Generate README  - Scan project and create a new README.md", value="generate"),
                questionary.Choice("Update README    - Update an existing Dokugen-generated README", value="update"),
                questionary.Choice("Exit", value="exit"),
            ],
        ).ask()

        if action == "exit" or action is None:
            console.print("[yellow]Goodbye![/yellow]")
            return

        class Args:
            template = None
            overwrite = True

        if action == "generate":
            cmd_generate(Args())
        elif action == "update":
            cmd_update(Args())

    else:
        args = parser.parse_args()
        if args.command == "generate":
            cmd_generate(args)
        elif args.command == "update":
            cmd_update(args)
        elif args.command == "compose-docker":
            cmd_compose_docker(args)
        else:
            parser.print_help()


if __name__ == "__main__":
    main()
